"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const core$1 = require("./core.cjs");
const utils = require("./utils.cjs");
const encoding = require("./encoding.cjs");
const lodashEs = require("lodash-es");
const react = require("react");
const reactStore = require("@tanstack/react-store");
const typeBasedOptions = {
  string: {
    ...encoding.stringSerializer
  },
  number: {
    ...encoding.numberSerializer
  },
  boolean: {
    ...encoding.booleanSerializer
  },
  datetime: {
    ...encoding.datetimeSerializer
  },
  date: {
    ...encoding.dateSerializer
  }
};
class InternalBuilder {
  constructor(state) {
    this.state = state;
    this.customSetter = null;
  }
  withSerializer(serializer) {
    this.state = {
      ...this.state,
      ...serializer
    };
    return this;
  }
  validate(validator) {
    this.state = {
      ...this.state,
      ...validator
    };
    return this;
  }
  withDefault(value) {
    this.state = {
      ...this.state,
      defaultValue: value
    };
    return this;
  }
  withCustomSetter(transformer) {
    this.customSetter = transformer;
    return this;
  }
}
const createInternalBuilder = (key, value) => {
  const options = typeBasedOptions[key] || {};
  const listOptions = key === "list" ? encoding.createListSerializer(value) : {};
  const defaultSerializers = {
    encode: encoding.defaultEncoder,
    decode: encoding.defaultDecoder
  };
  const state = {
    onError: () => {
    },
    validate: () => true,
    defaultValue: null,
    updateType: "replaceIn",
    ...defaultSerializers,
    ...options,
    ...value || {},
    ...listOptions
  };
  return new InternalBuilder(state);
};
const p = new Proxy({}, {
  get(_target, prop) {
    return (value) => {
      const key = prop;
      return createInternalBuilder(key, value);
    };
  }
});
const getWithPrefix = (prefix, prop) => {
  return prefix ? `${prefix}-${prop}` : prop;
};
const buildSingle = ({ builder, params, prop, globalOptions }) => {
  const { useParamSet, useParam } = params;
  const { state, customSetter } = builder;
  const typedOptions = state;
  const createSetter = (setter) => {
    return customSetter ? customSetter({ setter }) : setter;
  };
  return {
    useSet: (options) => {
      const memoedOptions = utils.useMemoOptions(typedOptions, globalOptions, options);
      const paramName = getWithPrefix(memoedOptions.prefix, prop);
      const setter = useParamSet(paramName, memoedOptions);
      return react.useMemo(() => createSetter(setter), [setter]);
    },
    use: (options) => {
      const memoedOptions = utils.useMemoOptions(typedOptions, globalOptions, options);
      const paramName = getWithPrefix(memoedOptions.prefix, prop);
      const [res, setter] = useParam(paramName, memoedOptions);
      const setterTransformed = react.useMemo(() => createSetter(setter), [setter]);
      return [
        res,
        setterTransformed
      ];
    }
  };
};
let core = null;
const init = lodashEs.once(() => {
  core = core$1.create();
});
function create(schema, globalOptions) {
  init();
  let internalGlobalOptions = globalOptions;
  const proxy = new Proxy({}, {
    get(_target, prop) {
      if (prop === "withOptions") {
        return (globalOptions2) => {
          internalGlobalOptions = {
            ...internalGlobalOptions,
            ...globalOptions2
          };
          return proxy;
        };
      }
      if (prop === "build") {
        return (values) => {
          const params = Object.entries(values).map(([key, value]) => {
            const schemaKey = utils.camelCaseToKebab(key);
            const builder2 = schema[schemaKey];
            const { defaultValue, encode } = builder2.state;
            if (value === utils.getValue(defaultValue)) {
              return void 0;
            }
            const encodedValue = encoding.encodeParam(value, encode);
            return `${key}=${encodedValue}`;
          }).filter(Boolean);
          return params.join("&");
        };
      }
      if (prop === "batch") {
        return (fn) => {
          const { api } = core;
          reactStore.batch(() => {
            api.batch(fn);
          });
        };
      }
      const schemaField = utils.camelCaseToKebab(prop);
      const builder = schema[schemaField];
      return buildSingle({
        builder,
        params: core,
        prop,
        globalOptions: internalGlobalOptions
      });
    }
  });
  return proxy;
}
function createSingle(prop, definition) {
  init();
  const builder = definition;
  return buildSingle({
    builder,
    params: core,
    prop
  });
}
exports.create = create;
exports.createSingle = createSingle;
exports.p = p;
//# sourceMappingURL=params.cjs.map
