import { useCallback, useMemo } from "react";
import { useStore, Store } from "@tanstack/react-store";
import { isEqual } from "lodash-es";
import { isClient, useSmartValue, isParamsTransition, getValue, paramsTransitioning } from "./utils.js";
import { encodeParam, decodeParam } from "./encoding.js";
import { useContextApi } from "./use-api.js";
import { withBatch, defaultApi, dummyApi } from "./api.js";
const create = () => {
  let isInitialized = false;
  let paramsStore = null;
  let api = null;
  const getLatestParams = (api2) => {
    const params = new URLSearchParams(api2.getSearch());
    const paramsMap = {};
    for (const [key, value] of params) {
      paramsMap[key] = value;
    }
    return paramsMap;
  };
  const init = (contextApi, isClient2) => {
    if (isClient2 && isInitialized) {
      return;
    }
    isInitialized = true;
    api = withBatch(contextApi ?? (isClient2 ? defaultApi() : dummyApi));
    if (isClient2) {
      api.registerListener((state) => {
        if (isParamsTransition(state)) {
          return;
        }
        paramsStore.setState(() => getLatestParams(api));
      });
      paramsStore = new Store(getLatestParams(api));
    }
  };
  const decodeWithDefault = (value, defaultValue, {
    decode,
    onError,
    validate
  }) => {
    if (value === void 0) {
      return defaultValue;
    }
    const internalOnError = (decoded) => {
      const onErrorValue = onError == null ? void 0 : onError(value, decoded);
      return onErrorValue ?? defaultValue;
    };
    try {
      const decoded = decodeParam(value, decode);
      const isValid = validate ? validate(decoded) : true;
      if (isValid) {
        return decoded;
      } else {
        return internalOnError(decoded);
      }
    } catch {
      return internalOnError();
    }
  };
  const useDefaultValue = ({ defaultValue }) => {
    const internalDefaultValue = useMemo(() => {
      return getValue(defaultValue);
    }, [defaultValue]);
    return useSmartValue(internalDefaultValue);
  };
  const useParamGet = (paramName, options) => {
    const contextApi = useContextApi();
    init(contextApi, isClient);
    const internalDefaultValue = useDefaultValue(options);
    const value = isClient ? useStore(paramsStore, (s) => s[paramName]) : getLatestParams(api)[paramName];
    const decodedValue = useMemo(() => decodeWithDefault(value, internalDefaultValue, options), [value, internalDefaultValue, options]);
    return useSmartValue(decodedValue);
  };
  const useParamSet = (paramName, options) => {
    const contextApi = useContextApi();
    init(contextApi, isClient);
    const internalDefaultValue = useDefaultValue(options);
    return useCallback((value, state) => {
      if (!isClient) {
        throw new Error("Cannot use react params on the server");
      }
      const { updateType = "replaceIn", encode } = options;
      const currentValue = paramsStore.state[paramName];
      const internalValue = typeof value === "function" ? value(decodeWithDefault(currentValue, internalDefaultValue, options)) : value;
      const newValue = encodeParam(internalValue, encode);
      if (currentValue === newValue || currentValue === void 0 && isEqual(internalDefaultValue, value)) {
        return;
      }
      paramsStore.setState((s) => {
        return { ...s, [paramName]: newValue };
      });
      const search = api.getSearch();
      const clearAll = !updateType.endsWith("In");
      const isReplace = updateType.startsWith("replace");
      const searchParams = !clearAll ? new URLSearchParams(search) : new URLSearchParams();
      if (!isEqual(internalDefaultValue, value)) {
        searchParams.set(paramName, newValue);
      } else {
        searchParams.delete(paramName);
      }
      const newHref = `?${searchParams.toString()}`;
      const internalState = { [paramsTransitioning]: true, ...state };
      if (isReplace) {
        api.replaceState(newHref, internalState);
      } else {
        api.pushState(newHref, internalState);
      }
    }, [paramName, internalDefaultValue, options]);
  };
  const useParam = (paramName, options) => {
    const value = useParamGet(paramName, options);
    const setter = useParamSet(paramName, options);
    return [value, setter];
  };
  return {
    useParamGet,
    useParamSet,
    useParam,
    paramsStore,
    api
  };
};
export {
  create
};
//# sourceMappingURL=core.js.map
